function [ measures ] = evaluate_tables_pv( tables )
%
% EVALUATE_TABLES  Evaulate 2 by 2 contigency tables for a variety of measures
%        
% Evaluate each of a collection of 2 by 2 contingency tables according to 
% a variety of measures. The contingency table looks as follows:
%
%                   q (cases)       not q (controls)  row totals
%                 ----------------------------------------------
%            p     f11 (a)          f10 (b)            f1plus
%        not p     f01 (c)          f00 (d)            f0plus
%                 ----------------------------------------------
% column totals   fplus1 (cases)    fplus0 (controls)    N
%
% In terms of other common notation, a = f11, b = f10, c = f01, and d = f00.
% It is assumed that tables is a four by number of tables arrays, is passed 
% in the order f11, f10, f01, f00 or a b c d.
% 
% The measures array is a number of table by number of measures array 
% that gives the measure for each table. The labels of the measures are
% also returned. If the use_bonferroni flag is set to 1, then
% the pvalue is adjusted by multiplying it by the number of tables and
% taking the min with 1. This reduces the p-values to account for multiple
% hypothesis testing. If sort_flag is set to 1, then the results are sorted
% in decreasing order by -log( pvalue ). the table labels are reordered to
% match this sorting.
%
% [ measures measure_labels table_labels ] = evaluate_tables( tables, table_labels, num_tests, sort_flag );

% Default sort flag and the number of tests

if nargin == 1
    table_labels = repmat({'aa'},1,size(tables,1));
    num_tests = 1;
    sort_flag = 0;
end

if nargin == 2
    num_tests = 1;
    sort_flag = 0;
end

[ m n ] = size( table_labels );
if m == 1
   table_labels = table_labels';
end

% Set up the list of measures.
labels = { 
 'OR', ...
 '-log10(pvalue)',... 
 'a', ...
 'b', ...
 'c', ...
 'd', ...
};
measure_labels = labels;

num_measures = length( labels );
num_tables = size( tables, 1 );
measures = zeros( num_tables, num_measures );

f11 = tables( :, 1 );
f10 = tables( :, 2 );
f01 = tables( :, 3 );
f00 = tables( :, 4 );

f1plus = f11 + f10;
f0plus = f01 + f00;
fplus1 = f11 + f01;
fplus0 = f10 + f00;
N = f1plus + f0plus;


% Compute OR
k = find( strcmpi( labels,'OR'));
num = (f11+1).* (f00+1);
denom = (f10+1) .* (f01+1);
i = denom ~= 0;
j = denom == 0;
jj = num == 0;
measures( i, k ) = num( i )  ./ denom( i );
measures( j, k ) = inf;
measures( jj, k ) = 0;

% Compute -log10 pvalue of the table
k = strcmpi( labels,'-log10(pvalue)' );
pvalue12 = zeros( num_tables, 1 );
pvalue34 = zeros( num_tables, 1 );
pvalue1 = zeros( num_tables, 1 );
pvalue2 = zeros( num_tables, 1 );
pvalue3 = zeros( num_tables, 1 );
pvalue4 = zeros( num_tables, 1 );
for i = 1 : num_tables
    if mod( i, 100 ) == 0
       % disp( i );
    end
    pvalue1( i ) = ...
        1 - hygecdf( f11( i ) - 1, ...
                     N( i ), ...
                     fplus1( i ), ...
                     f1plus( i ) );
    pvalue2( i ) = ...
            hygecdf( f10( i ), ...
                     N( i ), ...
                     fplus0( i ), ...
                     f1plus( i ) );
end

% Compute the left tail probability
for i = 1 : num_tables
    if mod( i, 100 ) == 0
       % disp( i );
    end
    pvalue3( i ) = ...
            hygecdf( f11( i ), ...
                     N( i ), ...
                     fplus1( i ), ...
                     f1plus( i ) );
    pvalue4( i ) = ...
        1 - hygecdf( f10( i ) - 1, ...
                     N( i ), ...
                     fplus0( i ), ...
                     f1plus( i ) );
end

% Pick the more accurate value for the right tail
i = ( pvalue1 >= 0.5 );
j = ( pvalue1 < 0.5 ); 
mask = max( [ pvalue1, pvalue2 ], [], 2 )  == 1;
imask = i & mask;
inotmask = i & ~mask;
pvalue12( imask ) = min( [ pvalue1( imask ), pvalue2( imask ) ], [], 2 );
pvalue12( inotmask ) = max( [ pvalue1( inotmask ), pvalue2( inotmask ) ], [], 2 );
mask = min( [ pvalue1, pvalue2 ], [], 2 ) == 0;
jmask = j & mask;
jnotmask = j & ~mask;
pvalue12( jmask ) = max( [ pvalue1( jmask ), pvalue2( jmask ) ], [], 2 );
pvalue12( jnotmask ) = min( [ pvalue1( jnotmask ), pvalue2( jnotmask ) ], [], 2 );

% Pick the more accurate value for the left tail
i = ( pvalue3 >= 0.5 );
j = ( pvalue3 < 0.5 ); 
mask = max( [ pvalue3, pvalue4 ], [], 2 )  == 1; 
imask = i & mask;
inotmask = i & ~mask;
pvalue34( imask ) = min( [ pvalue3( imask ), pvalue4( imask ) ], [], 2 );
pvalue34( inotmask ) = max( [ pvalue3( inotmask ), pvalue4( inotmask ) ], [], 2 );
mask = min( [ pvalue3, pvalue4 ], [], 2 )  == 0;
jmask = j & mask;
jnotmask = j & ~mask;
pvalue34( jmask ) = max( [ pvalue3( jmask ), pvalue4( jmask ) ], [], 2 );
pvalue34( jnotmask ) = min( [ pvalue3( jnotmask ), pvalue4( jnotmask ) ], [], 2 );

% Compute the overall pvalue
pvalue = min( pvalue12, pvalue34 );
pvalue = min( pvalue, 1 );
i = ( pvalue ~= 0 );
j = ( pvalue == 0 );
if num_tests > 1
    measures( i, k ) = -log10( min( pvalue( i ) * num_tests, 1 ) );
else
    measures( i, k ) = -log10( pvalue( i ) );
end    
measures( j, k ) = inf;

% Record entries of contingency table
k = strcmpi( labels,'a' );
measures( :, k ) = f11;
k = strcmpi( labels,'b' );
measures( :, k ) = f10;
k = strcmpi( labels,'c' );
measures( :, k ) = f01;
k = strcmpi( labels,'d' );
measures( :, k ) = f00;

% If indicated, sort the results according to p-value, then odds-ratio
if sort_flag
    k1 = find( strcmpi( labels,'OR'));
    k2 = find( strcmpi( labels,'-log10(pvalue)'));
    [ measures sort_indices ] = sortrows( measures, [ k2 k1 ] );

    % Make left tail values negative
    %left_tail = left_tail( sort_indices );
    %measures( :, k2 ) = measures( :, k2 ) .* (-left_tail + ~left_tail);

    % Flip the measures and the table labels so that highest pvalues are at the top
    measures = flipud( measures );
    table_labels = flipud( table_labels( sort_indices ) );
    sort_indices = flipud( sort_indices );
end

measures = measures(:,2);
