function from_plink2mat(rawfile, infocols, nsubj)


% FUNCTION: from_plink2mat(rawfile, infocols, nsubj)
%
% This function converts .raw plink files into .mat files.
% It extracts all information from the .raw file and separates 
% the genotype information from the rest. It saves it all into
% a <filename>.mat file. The .mat file consists of the following 
% variables:
% - infoheaders - headers of non-genotype info from plink file
% - sub_info - non-genotype info about subjects
% - snpRSNums - snp names
% - trans_data - genotype information
%
% INPUT: 
% rawfile - plink .raw file
% infocols - number of non-genotype columns (columns before 'rs' cols start)
% nsubj - number of subjects in the file
%
% sample run: from_plink2mat('Recode_all_chr.raw', 6, 270);


fid = fopen(rawfile, 'r');
colheaders = fgetl(fid);
[token remain] = strtok(colheaders, ' ');
dim = 1;
while(~isempty(remain))
   [token remain] = strtok(remain, ' ');
   dim = dim + 1;
end
nsnps = dim - infocols;
colheaders = textscan(colheaders, '%s', dim);
colheaders = colheaders{1};

infoheaders = colheaders(1:infocols);
snpRSNums = colheaders((infocols+1) : end);

sub_info = cell(nsubj, infocols);
trans_data = cell(nsubj, nsnps);

for i = 1:nsubj
    fline = fgetl(fid);
    subject = textscan(fline, '%s', dim);
    subject = subject{1};
    sub_info (i, :) = subject(1:infocols);
    trans_data (i, :) = subject((infocols+1) : end);
end

trans_data(strcmp('NA', trans_data)) = {'-1'};
trans_data = cellfun(@str2num, trans_data);

save (sprintf('%s.mat', rawfile), 'trans_data', 'sub_info', 'infoheaders', 'snpRSNums','-v7.3');
fclose(fid);
